import ctypes

from . import list_to_array, array_to_list
from . import aliases

se_clear_bad_flags = aliases['clear_bad_flags']

# this function is different just enough, so that the regular pattern of implementing functions won't work well.
def clear_bad_flags(complement, flag):
    """ 
        Sets all flags to False. If complement is True, then takes inverse of all flags.
        
        Args:
            complement = Whether to complement masks or not
            flag = Input masks that are to be set to all False or complemented.

        Returns:
          Numpy masked array: Contains an array of data, mask, and fill_value of results.

        Throws:
          ValueError: if input_list and input_boundary_mask are not equal in size,
    """


    # set return type and arg types
    se_clear_bad_flags.restype = None
    se_clear_bad_flags.argtypes = [ctypes.c_bool, ctypes.POINTER(ctypes.c_bool), ctypes.POINTER(ctypes.c_bool), ctypes.c_size_t]

    data_length = len(flag)

    complement_bool = ctypes.c_bool(complement)
    flagArray = list_to_array(flag, ctypes.c_bool)
    complementArray = list_to_array([False]*data_length, ctypes.c_bool)
    nGates_int = ctypes.c_size_t(data_length)

    # run C function, output_array is updated with despeckle results
    se_clear_bad_flags(
        complement_bool,
        flagArray,
        complementArray,
        nGates_int
    )

    # convert ctypes array to python list
    output_list = array_to_list(complementArray, data_length)

    # returns the new data and masks packaged in an object
    return output_list

def main():
    complement = False
    flag = [True, True, True, True]
    clear_bad_flags(complement, flag)


if __name__ == "__main__":
    main()



# Import all functions that would be used in user space.



from .solo_functions.solo_despeckle import despeckle, despeckle_masked
from .solo_functions.solo_ring_zap import ring_zap, ring_zap_masked
from .solo_functions.solo_threshold import threshold, threshold_masked
from .solo_functions.solo_flag_glitches import flag_glitches, flag_glitches_masked
from .solo_functions.solo_flag_freckles import flag_freckles, flag_freckles_masked
from .solo_functions.solo_forced_unfolding import forced_unfolding, forced_unfolding_masked
from .solo_functions.solo_unfold_first_good_gate import unfold_first_good_gate, unfold_first_good_gate_masked
from .solo_functions.solo_unfold_local_wind import unfold_local_wind, unfold_local_wind_masked
from .solo_functions.solo_radial_shear import radial_shear,radial_shear_masked
from .solo_functions.solo_rain_rate import rain_rate, rain_rate_masked
from .solo_functions.solo_remove_ac_motion import remove_ac_motion, remove_ac_motion_masked
from .solo_functions.solo_remove_storm_motion import remove_storm_motion, remove_storm_motion_masked
# from .solo_functions.solo_fix_vortex_vels import fix_vortex_vels, fix_vortex_vels_masked
from .solo_functions.solo_merge_fields import merge_fields, merge_fields_masked
from .solo_functions.flags.solo_assign_value import assign_value, assign_value_masked
from .solo_functions.flags.solo_assert_bad_flags import assert_bad_flags, assert_bad_flags_masked
from .solo_functions.flags.solo_bad_flags_logic import bad_flags_logic, bad_flags_logic_masked
from .solo_functions.flags.solo_copy_bad_flags import copy_bad_flags, copy_bad_flags_masked
from .solo_functions.flags.solo_flagged_add import flagged_add, flagged_add_masked
from .solo_functions.flags.solo_set_bad_flags import set_bad_flags, set_bad_flags_masked
from .solo_functions.flags.solo_clear_bad_flags import clear_bad_flags

from .enums import Where, Logical


import ctypes

from . import run_solo_function
from . import DataPair, masked_op
from . import aliases

se_assign_value = aliases['assign_value']


def assign_value(input_list_data, bad, constant, bad_flag_mask, dgi_clip_gate=None, boundary_mask=None):
    """ 
        For masked values, assign it to the defined "constant" value
        
        Args:
            input_list_data: Input float list
            bad: Float representing bad value.
            constant: Float value representing what masked values should become
            bad_flag_mask: A mask for input_list marking good or bad values.
            (optional) dgi_clip_gate: An integer determines the end of the ray (default: length of input_list)
            (optional) boundary_mask: Defines region over which operations will be done. (default: all True).

        Returns:
          Numpy masked array: Contains an array of data, mask, and fill_value of results.

        Throws:
          ValueError: if input_list and input_boundary_mask are not equal in size,

    """

    args = {
        "constant" : DataPair.DataTypeValue(ctypes.c_float, constant),
        "data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), input_list_data),
        "newData" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), None),
        "nGates" : DataPair.DataTypeValue(ctypes.c_size_t, None),
        "dgi_clip_gate" : DataPair.DataTypeValue(ctypes.c_size_t, dgi_clip_gate),
        "boundary_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), boundary_mask),
        "bad_flag_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), bad_flag_mask),
    }

    return run_solo_function(se_assign_value, args)


def assign_value_masked(masked_array, constant, boundary_mask=None):
    """ 
        For masked values, assign it to the defined "constant" value
        
        Args:
            masked_array: A numpy masked array data structure,
            constant: Float value representing what masked values should become

        Returns:
            Numpy masked array

        Throws:
            ModuleNotFoundError: if numpy is not installed
            AttributeError: if masked_array arg is not a numpy masked array.
    """
    return masked_op.masked_func(assign_value, masked_array, constant, boundary_mask = boundary_mask, usesBadFlags=True)



import ctypes

from . import Where
from . import run_solo_function
from . import DataPair, masked_op
from . import aliases

se_bad_flags_logic = aliases['bad_flags_logic']

def bad_flags_logic(input_list_data, bad, where, logical, scaled_thr1, scaled_thr2, bad_flag_mask, dgi_clip_gate=None, boundary_mask=None):
    """ 
        Performs a logical operation (LOGICAL enum) between bad_flag_mask and a value based on the WHERE enum.
        Equivalent to:
        flags[i] = (value[i] > WHERE) {logical operand} (bad_flag_mask[i]):

        Args:
            input_list_data: A list containing float data,
            bad: A float that represents a missing/invalid data point,
            where: A 'Where' enum, ABOVE(0), BELOW(1), BETWEEN(2) [Note: Not inclusive]
            logical: A 'Logical' enum, AND(0), OR(1), XOR(2)
            scaled_thr1: Lower bound threshold
            scaled_thr2: Upper bound threshold
            (optional) dgi_clip_gate: An integer determines the end of the ray (default: length of input_list)
            (optional) boundary_mask: Defines region over which operations will be done. (default: all True).

        Returns:
          Numpy masked array: Contains an array of data, mask, and fill_value of results.

        Throws:
          ValueError: if input_list and input_boundary_mask are not equal in size,

    """

    if isinstance(where, Where):
        where = where.value

    if not isinstance(where, int):
        raise ValueError(f"Expected integer or Where enum for 'where' parameter, received {type(where)}")

    args = {
        "scaled_thr1" : DataPair.DataTypeValue(ctypes.c_float, scaled_thr1),
        "scaled_thr2" : DataPair.DataTypeValue(ctypes.c_float, scaled_thr2),
        "where" : DataPair.DataTypeValue(ctypes.c_int, where),
        "logical" : DataPair.DataTypeValue(ctypes.c_int, logical),
        "data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), input_list_data),
        "nGates" : DataPair.DataTypeValue(ctypes.c_size_t, None),
        "bad" : DataPair.DataTypeValue(ctypes.c_float, bad),
        "dgi_clip_gate" : DataPair.DataTypeValue(ctypes.c_size_t, dgi_clip_gate),
        "boundary_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), boundary_mask),
        "flag" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), bad_flag_mask),
        "bad_flag_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), bad_flag_mask),
    }

    return run_solo_function(se_bad_flags_logic, args)


def bad_flags_logic_masked(masked_array, where, logical, scaled_thr1, scaled_thr2, boundary_mask=None):
    """ 
        Performs a logical operation (LOGICAL enum) between bad_flag_mask and a value based on the WHERE enum.
        
        Args:
            masked_array: A numpy masked array data structure,
            where: A 'Where' enum, ABOVE(0), BELOW(1), BETWEEN(2) [Note: Not inclusive]
            logical: A 'Logical' enum, AND(0), OR(1), XOR(2)
            scaled_thr1: Lower bound threshold
            scaled_thr2: Upper bound threshold

        Returns:
            Numpy masked array

        Throws:
            ModuleNotFoundError: if numpy is not installed
            AttributeError: if masked_array arg is not a numpy masked array.
    """
    return masked_op.masked_func(bad_flags_logic, masked_array, where, logical, scaled_thr1, scaled_thr2, boundary_mask = boundary_mask, usesBadFlags=True)



from enum import Enum

# Used by threshold_field, set_bad_flags, bad_flags_logic
class Where(Enum):
    ABOVE = 0
    BELOW = 1
    BETWEEN = 2
    
# Used by bad_flags_logic
class Logical(Enum):
    AND = 0
    OR = 1
    XOR = 2



import ctypes

from ..c_wrapper.run_solo import run_solo_function
from ..c_wrapper import DataPair, masked_op
from ..c_wrapper.function_alias import aliases
from ..enums import Where

se_threshold = aliases['threshold_field']

def threshold(input_list_data, threshold_list_data, bad, where, scaled_thr1, scaled_thr2, dgi_clip_gate=None, thr_missing=None, first_good_gate=0, boundary_mask=None):
    """
        Performs a threshold comparison on two lists of floats. If threshold_list_data has values ABOVE, BELOW, or BETWEEN the threshold values, 
        then those values are masked for input_list_data.

        Args:
            input_list: A list containing float data.
            thr_list: The referenced list for threshold
            bad: A float that represents a missing/invalid data point for input_list.
            where: A 'Where' enum, ABOVE(0), BELOW(1), BETWEEN(2)
            scaled_thr1: Lower bound threshold
            scaled_thr2: Upper bound threshold
            (optional) dgi_clip_gate: An integer determines the end of the ray (default: length of input_list)
            (optional) thr_missing: A float that represents a missing/invalid data point for thr_list (default: same value as bad)
            (optional) first_good_gate: Marks the index of the first "good" value in the input_list (default: 0) 
            (optional) boundary_mask: this is the masked region bool list where the function will perform its operation (default: all True, so operation performed on entire region).

        Returns:
            Numpy masked array: Contains an array of data, mask, and fill_value of results.


    """

    args = {
        "where" : DataPair.DataTypeValue(ctypes.c_int, where),
        "scaled_thr1" : DataPair.DataTypeValue(ctypes.c_float, scaled_thr1),
        "scaled_thr2" : DataPair.DataTypeValue(ctypes.c_float, scaled_thr2),
        "first_good_gate" : DataPair.DataTypeValue(ctypes.c_int, first_good_gate),
        "data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), input_list_data),
        "thr_data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), threshold_list_data),
        "nGates" : DataPair.DataTypeValue(ctypes.c_size_t, None),
        "newData" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), None),
        "bad" : DataPair.DataTypeValue(ctypes.c_float, bad),
        "thr_bad" : DataPair.DataTypeValue(ctypes.c_float, thr_missing if thr_missing else bad),
        "dgi_clip_gate" : DataPair.DataTypeValue(ctypes.c_size_t, dgi_clip_gate),
        "boundary_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), boundary_mask),
        "bad_flag_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), boundary_mask)
    }

    return run_solo_function(se_threshold, args)


def threshold_masked(masked_array, threshold_array, where, scaled_thr1, scaled_thr2, boundary_mask=None):
    """ 
        Performs a threshold mask operation on a numpy masked array (a field)
        For values in threshold field that are ABOVE, BELOW, or BETWEEN a threshold value,
        mask the reference field at those values.
        
        Args:
            masked_array: A numpy masked array data structure,
            threshold_array: A numpy masked array data structure for referenced threshold,
            where: A 'Where' enum, ABOVE(0), BELOW(1), BETWEEN(2)
            scaled_thr1: Lower bound threshold
            scaled_thr2: Upper bound threshold
            
        Returns:
            Numpy masked array

        Throws:
            ModuleNotFoundError: if numpy is not installed
            AttributeError: if masked_array arg is not a numpy masked array.
    """

    return masked_op.masked_func(threshold, masked_array, where, scaled_thr1, scaled_thr2, boundary_mask = boundary_mask, second_masked_array=threshold_array)



import ctypes

from ..c_wrapper.run_solo import run_solo_function

from ..c_wrapper import DataPair, masked_op
from ..c_wrapper.function_alias import aliases

se_remove_ac_motion = aliases['remove_ac_motion']



def remove_ac_motion(input_list_data, bad, vert_velocity, ew_velocity, ns_velocity, ew_gndspd_corr, tilt, elevation, dds_radd_eff_unamb_vel, seds_nyquist_velocity, dgi_clip_gate=None, boundary_mask=None):
    """ 
        Performs a <TODO> operation on a list of data.
        
        Args:
            input_list: A list containing float data,
            bad: A float that represents a missing/invalid data point,
            vert_velocity: <TODO>
            ew_velocity: <TODO>
            ns_velocity: <TODO>
            ew_gndspd_corr: <TODO>
            tilt: <TODO>
            elevation: <TODO>
            dds_radd_eff_unamb_vel: <TODO>
            seds_nyquist_velocity: <TODO>
            (optional) dgi_clip_gate: An integer determines the end of the ray (default: length of input_list)
            (optional) boundary_mask: Defines region over which operations will be done. (default: all True).

        Returns:
          Numpy masked array: Contains an array of data, mask, and fill_value of results.

        Throws:
          ValueError: if input_list and input_boundary_mask are not equal in size,
    """

    args = {
        "vert_velocity" : DataPair.DataTypeValue(ctypes.c_float, vert_velocity),
        "ew_velocity" : DataPair.DataTypeValue(ctypes.c_float, ew_velocity),
        "ns_velocity" : DataPair.DataTypeValue(ctypes.c_float, ns_velocity),
        "ew_gndspd_corr" : DataPair.DataTypeValue(ctypes.c_float, ew_gndspd_corr),
        "tilt" : DataPair.DataTypeValue(ctypes.c_float, tilt),
        "elevation" : DataPair.DataTypeValue(ctypes.c_float, elevation),
        "data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), input_list_data),
        "newData" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), None),
        "nGates" : DataPair.DataTypeValue(ctypes.c_size_t, None),
        "bad" : DataPair.DataTypeValue(ctypes.c_float, bad),
        "dgi_clip_gate" : DataPair.DataTypeValue(ctypes.c_size_t, dgi_clip_gate),
        "dds_radd_eff_unamb_vel" : DataPair.DataTypeValue(ctypes.c_float, dds_radd_eff_unamb_vel),
        "seds_nyquist_velocity" : DataPair.DataTypeValue(ctypes.c_float, seds_nyquist_velocity),
        "boundary_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), boundary_mask),
    }

    return run_solo_function(se_remove_ac_motion, args)


def remove_ac_motion_masked(masked_array, vert_velocity, ew_velocity, ns_velocity, ew_gndspd_corr, tilt, elevation, dds_radd_eff_unamb_vel, seds_nyquist_velocity, boundary_mask=None):
    """ 
        Performs a <TODO> operation on a numpy masked array
        
        Args:
            masked_array: A numpy masked array data structure,
            vert_velocity: <TODO>
            ew_velocity: <TODO>
            ns_velocity: <TODO>
            ew_gndspd_corr: <TODO>
            tilt: <TODO>
            elevation: <TODO>
            dds_radd_eff_unamb_vel: <TODO>
            seds_nyquist_velocity: <TODO>

        Returns:
            Numpy masked array

        Throws:
            ModuleNotFoundError: if numpy is not installed
            AttributeError: if masked_array arg is not a numpy masked array.
    """


    return masked_op.masked_func(remove_ac_motion, masked_array, vert_velocity, ew_velocity, ns_velocity, ew_gndspd_corr, tilt, elevation, dds_radd_eff_unamb_vel, seds_nyquist_velocity, boundary_mask = boundary_mask)



import ctypes

from ..c_wrapper.run_solo import run_solo_function
from ..c_wrapper import DataPair, masked_op
from ..c_wrapper.function_alias import aliases

se_despeckle = aliases['despeckle']


def despeckle(input_list_data, bad, a_speckle, dgi_clip_gate=None, boundary_mask=None):
    """
        Performs a despeckle operation on a list of data (a single ray)

        Args:
            input_list_data: A list containing float data,
            bad: A float that represents a missing/invalid data point,
            a_speckle: An integer that determines the number of contiguous good data considered a speckle,
            (optional) dgi_clip_gate: An integer determines the end of the ray (default: length of input_list),
            (optional) boundary_mask: this is the masked region bool list where the function will perform its operation (default: all True, so operation performed on entire region).

        Returns:
            Numpy masked array: Contains an array of data, mask, and fill_value of results.

    """

    args = {
        "data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), input_list_data),
        "newData" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), None),
        "nGates" : DataPair.DataTypeValue(ctypes.c_size_t, None),
        "bad" : DataPair.DataTypeValue(ctypes.c_float, bad),
        "a_speckle" : DataPair.DataTypeValue(ctypes.c_size_t, a_speckle),
        "dgi_clip_gate" : DataPair.DataTypeValue(ctypes.c_size_t, dgi_clip_gate),
        "boundary_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), boundary_mask),
    }

    return run_solo_function(se_despeckle, args)


def despeckle_masked(masked_array, a_speckle, boundary_mask=None):
   return masked_op.masked_func(despeckle, masked_array, a_speckle, boundary_mask)
   


# void se_fix_vortex_vels(
# const float* data, 
# float* newData, 
# size_t nGates,
# const float* vs_data, 
# const float* vl_data,
#	float vs_xmitted_freq,
#	float vs_interpulse_time, float vl_interpulse_time,
#	float bad, size_t dgi_clip_gate, bool* boundary_mask);

import ctypes

from ..c_wrapper.run_solo import run_solo_function
from ..c_wrapper import DataPair, masked_op
from ..c_wrapper.function_alias import aliases

se_fix_vortex_vels = aliases['fix_vortex_vels']

def fix_vortex_vels(input_list_data, bad, vs_data, vl_data, vs_xmitted_freq, vs_interpulse_time, vl_interpulse_time, dgi_clip_gate=None, boundary_mask=None):
    """ 
        Performs a TODO on a list of data.
        
        Args:
            input_list: A list containing float data,
            bad: A float that represents a missing/invalid data point,
            vs_data: <TODO>
            vl_data: <TODO>
            vs_xmitted_freq: <TODO>
            vs_interpulse_time: <TODO>
            vl_interpulse_time: <TODO>
            (optional) dgi_clip_gate: An integer determines the end of the ray (default: length of input_list)
            (optional) boundary_mask: Defines region over which operations will be done. (default: all True).

        Returns:
          Numpy masked array: Contains an array of data, mask, and fill_value of results.

        Throws:
          ValueError: if input_list and input_boundary_mask are not equal in size,

    """

    args = {
        "data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), input_list_data),
        "newData" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), None),
        "nGates" : DataPair.DataTypeValue(ctypes.c_size_t, None),
        "vs_data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), vs_data),
        "vl_data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), vl_data),
        "vs_xmitted_freq" : DataPair.DataTypeValue(ctypes.c_float, vs_xmitted_freq),
        "vs_interpulse_time" : DataPair.DataTypeValue(ctypes.c_float, vs_interpulse_time),
        "vl_interpulse_time" : DataPair.DataTypeValue(ctypes.c_float, vl_interpulse_time),
        "bad" : DataPair.DataTypeValue(ctypes.c_float, bad),
        "dgi_clip_gate" : DataPair.DataTypeValue(ctypes.c_size_t, dgi_clip_gate),
        "boundary_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), boundary_mask),
    }

    return run_solo_function(se_fix_vortex_vels, args)


def fix_vortex_vels_masked(masked_array, vs_data, vl_data, vs_xmitted_freq, vs_interpulse_time, vl_interpulse_time, boundary_mask=None):
    """ 
        Performs a ring zap operation on a numpy masked array
        
        Args:
            masked_array: A numpy masked array data structure,
            vs_data: <TODO>
            vl_data: <TODO>
            vs_xmitted_freq: <TODO>
            vs_interpulse_time: <TODO>
            vl_interpulse_time: <TODO>
            km_between_gates: An integer representing the distance (in km) between gates

        Returns:
            Numpy masked array

        Throws:
            ModuleNotFoundError: if numpy is not installed
            AttributeError: if masked_array arg is not a numpy masked array.
    """

    return masked_op.masked_func(fix_vortex_vels, masked_array, vs_data, vl_data, vs_xmitted_freq, vs_interpulse_time, vl_interpulse_time, boundary_mask = boundary_mask)



import ctypes

from . import run_solo_function
from . import DataPair, masked_op
from . import aliases

se_flagged_add = aliases['flagged_add']

def flagged_add(input_list_data, bad, f_const, multiply, bad_flag_mask, dgi_clip_gate=None, boundary_mask=None):
    """ 
        On masked values, add f_const parameter to value in input_list_data.
        Or multiple f_const if multiply set to true.
        
        Args:
            input_list_data: A list containing float data,
            bad: A float that represents a missing/invalid data point,
            f_const: Float constant value,
            multiply: If set to true, masked values will be multiplied by f_const. Otherwise, they will be added.
            (optional) dgi_clip_gate: An integer determines the end of the ray (default: length of input_list)
            (optional) boundary_mask: Defines region over which operations will be done. (default: all True).

        Returns:
          Numpy masked array: Contains an array of data, mask, and fill_value of results.

        Throws:
          ValueError: if input_list and input_boundary_mask are not equal in size,

    """

    args = {
        "f_const" : DataPair.DataTypeValue(ctypes.c_float, f_const),
        "multiply" : DataPair.DataTypeValue(ctypes.c_bool, multiply),
        "data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), input_list_data),
        "newData" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), None),
        "nGates" : DataPair.DataTypeValue(ctypes.c_size_t, None),
        "bad" : DataPair.DataTypeValue(ctypes.c_float, bad),
        "dgi_clip_gate" : DataPair.DataTypeValue(ctypes.c_size_t, dgi_clip_gate),
        "boundary_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), boundary_mask),
        "flag" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), bad_flag_mask),
    }

    return run_solo_function(se_flagged_add, args)


def flagged_add_masked(masked_array, f_const, multiply, boundary_mask=None):
    """ 
        Performs a <TODO> operation on a numpy masked array
        
        Args:
            masked_array: A numpy masked array data structure,
            <TODO>

        Returns:
            Numpy masked array

        Throws:
            ModuleNotFoundError: if numpy is not installed
            AttributeError: if masked_array arg is not a numpy masked array.
    """
    return masked_op.masked_func(flagged_add, masked_array, f_const, multiply, boundary_mask = boundary_mask, usesBadFlags=True)



import ctypes
from ..c_wrapper.run_solo import run_solo_function

from ..c_wrapper import  DataPair, masked_op
from ..c_wrapper.function_alias import aliases

se_funfold = aliases['funfold']

def forced_unfolding(input_list_data, bad, nyquist_velocity, dds_radd_eff_unamb_vel, center, dgi_clip_gate=None, boundary_mask=None):
    """ 
       Forces all data points to fall within plus or minus the Nyquist
        
        Args:
            input_list: A list containing float data,
            bad: A float that represents a missing/invalid data point,
            nyquist_velocity: Float value obtained from the radar,
            dds_radd_eff_unamb_vel: Float value obtained from the radar, 
            center: <TODO>,
            (optional) dgi_clip_gate: An integer determines the end of the ray (default: length of input_list)
            (optional) boundary_mask: Defines region over which operations will be done. (default: all True).

        Returns:
          Numpy masked array: Contains an array of data, mask, and fill_value of results.

        Throws:
          ValueError: if input_list and input_boundary_mask are not equal in size,

    """

    args = {
        "data" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), input_list_data),
        "newData" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_float), None),
        "nGates" : DataPair.DataTypeValue(ctypes.c_size_t, None),
        "nyquist_velocity" : DataPair.DataTypeValue(ctypes.c_float, nyquist_velocity),
        "dds_radd_eff_unamb_vel" : DataPair.DataTypeValue(ctypes.c_float, dds_radd_eff_unamb_vel),
        "center" : DataPair.DataTypeValue(ctypes.c_float, center),
        "bad" : DataPair.DataTypeValue(ctypes.c_float, bad),
        "dgi_clip_gate" : DataPair.DataTypeValue(ctypes.c_size_t, dgi_clip_gate),
        "boundary_mask" : DataPair.DataTypeValue(ctypes.POINTER(ctypes.c_bool), boundary_mask),
    }

    return run_solo_function(se_funfold, args)


def forced_unfolding_masked(masked_array, nyquist_velocity, dds_radd_eff_unamb_vel, center, boundary_mask=None):
    """ 
       Forces all data points to fall within plus or minus the Nyquist
        
        Args:
            masked_array: A numpy masked array data structure,
            nyquist_velocity: Float value obtained from the radar,
            dds_radd_eff_unamb_vel: Float value obtained from the radar, 
            center: <TODO>,

        Returns:
            Numpy masked array

        Throws:
            ModuleNotFoundError: if numpy is not installed
            AttributeError: if masked_array arg is not a numpy masked array.
    """

    return masked_op.masked_func(forced_unfolding, masked_array, nyquist_velocity, dds_radd_eff_unamb_vel, center, boundary_mask = boundary_mask)
